# Generated by roxygen2: do not edit by hand

S3method(plot,assess_phregr)
S3method(print,BOINTable)
S3method(print,adaptDesign)
S3method(print,adaptDesign_multiarm)
S3method(print,adaptDesign_seamless)
S3method(print,assess_phregr)
S3method(print,design)
S3method(print,designANOVA)
S3method(print,designANOVAContrast)
S3method(print,designAgreement)
S3method(print,designEquiv)
S3method(print,designLogistic)
S3method(print,designMeanDiff)
S3method(print,designMeanDiffCarryover)
S3method(print,designMeanDiffCarryoverEquiv)
S3method(print,designMeanDiffEquiv)
S3method(print,designMeanDiffMMRM)
S3method(print,designMeanDiffXO)
S3method(print,designMeanDiffXOEquiv)
S3method(print,designMeanRatio)
S3method(print,designMeanRatioEquiv)
S3method(print,designMeanRatioXO)
S3method(print,designMeanRatioXOEquiv)
S3method(print,designOddsRatio)
S3method(print,designOddsRatioEquiv)
S3method(print,designOneMean)
S3method(print,designOneMultinom)
S3method(print,designOneProportion)
S3method(print,designOneSlope)
S3method(print,designOrderedBinom)
S3method(print,designPairedMeanDiff)
S3method(print,designPairedMeanDiffEquiv)
S3method(print,designPairedMeanRatio)
S3method(print,designPairedMeanRatioEquiv)
S3method(print,designPairedPropMcNemar)
S3method(print,designRepeatedANOVA)
S3method(print,designRepeatedANOVAContrast)
S3method(print,designRiskDiff)
S3method(print,designRiskDiffEquiv)
S3method(print,designRiskRatio)
S3method(print,designRiskRatioEquiv)
S3method(print,designRiskRatioFM)
S3method(print,designSlopeDiff)
S3method(print,designSlopeDiffMMRM)
S3method(print,designTwoMultinom)
S3method(print,designTwoOrdinal)
S3method(print,designTwoWayANOVA)
S3method(print,designUnorderedBinom)
S3method(print,designUnorderedMultinom)
S3method(print,designWilcoxon)
S3method(print,kmpower)
S3method(print,kmpower1s)
S3method(print,kmpowerequiv)
S3method(print,liferegr)
S3method(print,logisregr)
S3method(print,lrpower)
S3method(print,lrpowerequiv)
S3method(print,lrsim)
S3method(print,lrsim_bmTrtSel)
S3method(print,lrsim_mcpmod)
S3method(print,lrsim_multiarm)
S3method(print,lrsim_seamless)
S3method(print,mTPI2Table)
S3method(print,multiarm)
S3method(print,nbpower)
S3method(print,nbpower1s)
S3method(print,nbpowerequiv)
S3method(print,phregr)
S3method(print,rdsim_multiarm)
S3method(print,rdsim_seamless)
S3method(print,rmpower)
S3method(print,rmpower1s)
S3method(print,rmpowerequiv)
S3method(print,seamless)
export(BOINTable)
export(ClopperPearsonCI)
export(accrual)
export(adaptDesign)
export(adaptDesign_multiarm)
export(adaptDesign_seamless)
export(assess_phregr)
export(binary_tte_sim)
export(caltime)
export(corr_pfs_os)
export(covrmst)
export(dtpwexp)
export(errorSpent)
export(exitprob)
export(exitprob_multiarm)
export(exitprob_seamless)
export(fCERCer)
export(fCERNewBound)
export(fCERRej)
export(fCERStageBound)
export(fDefaultWgtmat)
export(fPCRej)
export(fPCStage1)
export(fPCStagewise)
export(fadjpbon)
export(fadjpdun)
export(fadjpsim)
export(finthyp)
export(float_to_fraction)
export(fmodmix)
export(fquantile)
export(fseqbon)
export(fstdmix)
export(fstp2seq)
export(ftrunc)
export(fwgtmat)
export(getADCI)
export(getADCI_multiarm)
export(getADCI_seamless)
export(getADRCI)
export(getAccrualDurationFromN)
export(getBound)
export(getBound_multiarm)
export(getBound_seamless)
export(getCI)
export(getCI_multiarm)
export(getCI_seamless)
export(getCP)
export(getCP_multiarm)
export(getCP_seamless)
export(getDesign)
export(getDesign2)
export(getDesignANOVA)
export(getDesignANOVAContrast)
export(getDesignAgreement)
export(getDesignEquiv)
export(getDesignFisherExact)
export(getDesignLogistic)
export(getDesignMeanDiff)
export(getDesignMeanDiffCarryover)
export(getDesignMeanDiffCarryoverEquiv)
export(getDesignMeanDiffEquiv)
export(getDesignMeanDiffMMRM)
export(getDesignMeanDiffXO)
export(getDesignMeanDiffXOEquiv)
export(getDesignMeanRatio)
export(getDesignMeanRatioEquiv)
export(getDesignMeanRatioXO)
export(getDesignMeanRatioXOEquiv)
export(getDesignOddsRatio)
export(getDesignOddsRatioEquiv)
export(getDesignOneMean)
export(getDesignOneMultinom)
export(getDesignOneProportion)
export(getDesignOneRateExact)
export(getDesignOneSlope)
export(getDesignOrderedBinom)
export(getDesignPairedMeanDiff)
export(getDesignPairedMeanDiffEquiv)
export(getDesignPairedMeanRatio)
export(getDesignPairedMeanRatioEquiv)
export(getDesignPairedPropMcNemar)
export(getDesignRepeatedANOVA)
export(getDesignRepeatedANOVAContrast)
export(getDesignRiskDiff)
export(getDesignRiskDiffEquiv)
export(getDesignRiskDiffExact)
export(getDesignRiskDiffExactEquiv)
export(getDesignRiskRatio)
export(getDesignRiskRatioEquiv)
export(getDesignRiskRatioExact)
export(getDesignRiskRatioExactEquiv)
export(getDesignRiskRatioFM)
export(getDesignSlopeDiff)
export(getDesignSlopeDiffMMRM)
export(getDesignTwoMultinom)
export(getDesignTwoOrdinal)
export(getDesignTwoWayANOVA)
export(getDesignUnorderedBinom)
export(getDesignUnorderedMultinom)
export(getDesignWilcoxon)
export(getDesign_multiarm)
export(getDesign_seamless)
export(getDurationFromNevents)
export(getNeventsFromHazardRatio)
export(getRCI)
export(hazard_pd)
export(hazard_sub)
export(hedgesg)
export(kmdiff)
export(kmest)
export(kmpower)
export(kmpower1s)
export(kmpowerequiv)
export(kmsamplesize)
export(kmsamplesize1s)
export(kmsamplesizeequiv)
export(kmstat)
export(kmsurv)
export(liferegr)
export(logisregr)
export(lrpower)
export(lrpowerequiv)
export(lrsamplesize)
export(lrsamplesizeequiv)
export(lrschoenfeld)
export(lrsim)
export(lrsim2e)
export(lrsim2e3a)
export(lrsim3a)
export(lrsim_bmTrtSel)
export(lrsim_mcpmod)
export(lrsim_multiarm)
export(lrsim_seamless)
export(lrsimsub)
export(lrstat)
export(lrtest)
export(mTPI2Table)
export(mnOddsRatioCI)
export(mnRateDiffCI)
export(mnRateRatioCI)
export(mnRiskDiffCI)
export(mnRiskRatioCI)
export(mtpwexp)
export(natrisk)
export(nbpower)
export(nbpower1s)
export(nbpowerequiv)
export(nbsamplesize)
export(nbsamplesize1s)
export(nbsamplesizeequiv)
export(nbstat)
export(nevent)
export(patrisk)
export(pbvnorm)
export(pevent)
export(phregr)
export(pmvnormr)
export(powerFisherExact)
export(powerOnePropExact)
export(powerOneRateExact)
export(powerRiskDiffExact)
export(powerRiskDiffExactEquiv)
export(powerRiskRatioExact)
export(powerRiskRatioExactEquiv)
export(ptpwexp)
export(pwexpcuts)
export(pwexploglik)
export(qmvnormr)
export(qtpwexp)
export(rdsim_multiarm)
export(rdsim_seamless)
export(remlOddsRatio)
export(remlRateDiff)
export(remlRateRatio)
export(remlRiskDiff)
export(remlRiskRatio)
export(repeatedPValue)
export(residuals_liferegr)
export(residuals_phregr)
export(riskDiffExactCI)
export(riskDiffExactPValue)
export(riskRatioExactCI)
export(riskRatioExactPValue)
export(rmdiff)
export(rmest)
export(rmpower)
export(rmpower1s)
export(rmpowerequiv)
export(rmsamplesize)
export(rmsamplesize1s)
export(rmsamplesizeequiv)
export(rmst)
export(rmstat)
export(rtpwexp)
export(runShinyApp_lrstat)
export(samplesizeFisherExact)
export(samplesizeOnePropExact)
export(samplesizeOneRateExact)
export(samplesizeRiskDiffExact)
export(samplesizeRiskDiffExactEquiv)
export(samplesizeRiskRatioExact)
export(samplesizeRiskRatioExactEquiv)
export(simon2stage)
export(simonBayesAnalysis)
export(simonBayesSim)
export(survQuantile)
export(survfit_phregr)
export(svdcpp)
export(updateGraph)
export(zph_phregr)
export(zstatOddsRatio)
export(zstatRateDiff)
export(zstatRateRatio)
export(zstatRiskDiff)
export(zstatRiskRatio)
importFrom(Rcpp,evalCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(RcppParallel,setThreadOptions)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_linewidth_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(parallel,detectCores)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(rlang,.data)
importFrom(shiny,shinyAppDir)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,dgamma)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,formula)
importFrom(stats,integrate)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optimize)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qbeta)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,reformulate)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,var)
useDynLib(lrstat, .registration = TRUE)
