% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mtpwexp}
\alias{mtpwexp}
\title{Mean and Variance of Truncated Piecewise Exponential Distribution}
\usage{
mtpwexp(piecewiseSurvivalTime = 0L, lambda = NA_real_, lowerBound = 0)
}
\arguments{
\item{piecewiseSurvivalTime}{A vector that specifies the starting time of
piecewise exponential survival time intervals. Must start with 0, e.g.,
\code{c(0, 6)} breaks the time axis into 2 event intervals:
\eqn{[0, 6)} and \eqn{[6, \infty)}.
Defaults to 0 for exponential distribution.}

\item{lambda}{A vector of hazard rates for the event. One for
each analysis time interval.}

\item{lowerBound}{The left truncation time point for the survival time.
Defaults to 0 for no truncation.}
}
\value{
A list with two components, one for the mean, and the other for
the variance of the truncated piecewise exponential distribution.
}
\description{
Obtains the mean and variance from a truncated piecewise
exponential distribution.
}
\examples{
mtpwexp(piecewiseSurvivalTime = c(0, 6, 9, 15),
        lambda = c(0.025, 0.04, 0.015, 0.007))

}
\author{
Kaifeng Lu, \email{kaifenglu@gmail.com}
}
