% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prints_getDesignProportions.R
\name{print.designOddsRatio}
\alias{print.designOddsRatio}
\title{Print Power and Sample Size Results for Two-Sample Odds Ratio}
\usage{
\method{print}{designOddsRatio}(x, ...)
}
\arguments{
\item{x}{The designOddsRatio object to print.}

\item{...}{Ensures that all arguments starting from "..." are named.}
}
\value{
A tabular printout of the design elements.
}
\description{
Prints the summary statistics from power calculation of
two-sample odds ratio.
}
\author{
Kaifeng Lu, \email{kaifenglu@gmail.com}
}
\keyword{internal}
