% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parameter_descriptions.R
\name{param_kMax}
\alias{param_kMax}
\title{Parameter Description: kMax}
\arguments{
\item{kMax}{The maximum number of stages.}
}
\description{
Parameter Description: kMax
}
\keyword{internal}
